#include "clock.h"
#include "key.h"

/* Current (expected) state of toggle bit */
static enum {
	TOGGLE_ON,
	TOGGLE_OFF,
	TOGGLE_UNKNOWN,
} key_toggle_state;

/* Translation from command codes to key codes */
static enum key_code key_translate(uint8_t command)
{
	enum key_code code = KEY_INVALID;
	
	switch (command) {
	case  0:  code = KEY_DIG0;  break;
	case  1:  code = KEY_DIG1;  break;
	case  2:  code = KEY_DIG2;  break;
	case  3:  code = KEY_DIG3;  break;
	case  4:  code = KEY_DIG4;  break;
	case  5:  code = KEY_DIG5;  break;
	case  6:  code = KEY_DIG6;  break;
	case  7:  code = KEY_DIG7;  break;
	case  8:  code = KEY_DIG8;  break;
	case  9:  code = KEY_DIG9;  break;
	case 37:  code = KEY_OK;    break;
	case 42:  code = KEY_TIME;  break;
	};
	
	return code;
}

bool key_on_data(const struct rc5_data *data)
{
	bool consumed = false;
	enum key_code code = KEY_INVALID;

	/* Check toggle state to see if this event must be translated to
	 * a new key stroke. This filters out repeated transmissions.
	 */
	switch (key_toggle_state) {
	case TOGGLE_ON:
		if (data->toggle == false) {
			code = key_translate(data->command);
			key_toggle_state = TOGGLE_OFF;
		}
		break;
	case TOGGLE_OFF:
		if (data->toggle == true) {
			code = key_translate(data->command);
			key_toggle_state = TOGGLE_ON;
		}
		break;
	default:
		/* Startup: use first event, from now on we know
		 * what can be expected as next toggle state
		 */
		code = key_translate(data->command);
		key_toggle_state = data->toggle ? TOGGLE_ON : TOGGLE_OFF;
		break;
	};
	
	/* If it was a key we know, pass it to the application */
	if (code != KEY_INVALID)
		consumed = clock_on_key(code);
	
	return consumed;
}

void key_init(void)
{
	/* At startup we don't know what the next toggle should be */
	key_toggle_state = TOGGLE_UNKNOWN;
}
